/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* Here's the calculation of the required shm block size. The Servicereport
 * has already one Backendstate in it, so we need to add (n-1) blocks to
 * make block->backendstate a linear array. */
#define REPORTSIZE(s) \
    sizeof(Servicereport) + ((s)->nbackend - 1) * sizeof(Backendstate)

void alloc_reporter (Service *s, int first) {
    int shmid;
    struct shmid_ds shmbuf;

    /* Get an ID to work on, depending on the create mode.
       Get a corresponding semaphore too. */
    msg ("Obtaining reporting memory for service %s (key %d)",
	 s->name, s->shmkey);

    if (first) {
	/* First time 'round. We're allocating for the first time here.*/
	if ( (shmid = shmget (s->shmkey, REPORTSIZE(s), 
			      opt.shmperm | IPC_CREAT) ) >= 0)
	    msg ("Created new shm at id %d", shmid);
	else 
	    error ("Cannot create shared memory for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));

	/* Make sure that we're the owner of the segment. */
	if (shmctl (shmid, IPC_STAT, &shmbuf) == -1)
	    error ("Failed to get status of shared memory: %s",
		   strerror(errno));
	shmbuf.shm_perm.uid = getuid();
	shmbuf.shm_perm.gid = getgid();
	if (shmctl (shmid, IPC_SET, &shmbuf) == -1)
	    error ("Failed to set status of shared memory: %s",
		   strerror(errno));
	
	if ( (semid = semget (s->shmkey, 1, opt.shmperm | IPC_CREAT)) >= 0 )
	    msg ("Created new sem at id %d", semid);
	else
	    error ("Cannot create semaphore for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));
    } else {
	/* Second time 'round. We're getting a handle on previously
	 * created shared memory and semaphores. */
	if ( (shmid = shmget (s->shmkey, REPORTSIZE(s), opt.shmperm) ) >= 0 )
	    msg ("Got existing shm at id %d", shmid);
	else
	    error ("Cannot get shared memory for service %s, key %d: %s. "
		   "Maybe crossroads isn't running, or the configuration "
		   "has changed.",
		   s->name, s->shmkey, strerror(errno));
	if ( (semid = semget (s->shmkey, 1, opt.shmperm)) >= 0 )
	    msg ("Got existing sem at id %d", semid);
	else
	    error ("Cannot get semaphore for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));
    }

    /* Attach to our memory segment. */
    if ( ((servicereport = shmat (shmid, 0, 0))) == (void *) -1 )
	error ("Cannot attach shared memory for service %s: %s",
	       s->name, strerror(errno));
}
