/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

void check_service (Service s) {
    int i;

    /* Check for port duplicates, bindto duplicates */
    for (i = 0; i < nservice; i++) {
	if (service[i].port == s.port) {
	    if (service[i].bind && s.bind &&
		!strcmp(service[i].bind, s.bind))
		error ("Services %s and %s cannot bind to same IP %s and "
		       "port %d",
		       service[i].name, s.name,
		       s.bind, s.port);
	    if (!service[i].bind && s.bind)
		error ("Services %s and %s use the same port %d, but "
		       "%s does not bind to a specific IP",
		       service[i].name, s.name,
		       s.port, service[i].name);
	    if (service[i].bind && !s.bind)
		error ("Services %s and %s use the same port %d, but "
		       "%s does not bind to a specific IP",
		       service[i].name, s.name,
		       s.port, s.name);
	    if (!service[i].bind && !s.bind)
		error ("Services %s and %s use the same port without "
		       "binding to an IP",
		       service[i].name, s.name);
	}
    }
    
    /* Verify the service description, port */
    if (!s.port)
	error ("Service %s lacks a port",
	       s.name);
    if (!s.nbackend)
	error ("Service %s lacks back ends",
	       s.name);
    if (s.rev_interval && s.check_interval)
	error ("Service %s has both a revivinginterval and "
	       "a checkinterval", s.name);
    
    /* HTTP modification directives only with "type http" */
    if (s.type != type_http) {
	for (i = 0; i < s.nbackend; i++) {
	    if (s.backend[i].stickycookie &&
		*s.backend[i].stickycookie)
		error ("Backend %s of service %s: sticky cookies "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].naddclientheader)
		error ("Backend %s of service %s: addclientheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].nsetclientheader)
		error ("Backend %s of service %s: setclientheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].nappendclientheader)
		error ("Backend %s of service %s: appendclientheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].naddserverheader)
		error ("Backend %s of service %s: addserverheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].nsetserverheader)
		error ("Backend %s of service %s: setserverheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	    if (s.backend[i].nappendserverheader)
		error ("Backend %s of service %s: appendserverheader "
		       "only allowed in 'type http'",
		       s.backend[i].name, s.name);
	}
    }
}
