/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

static char *timestr (double nsec) {
    static char *buf;
    char *tmp;
    
    free (buf);
    buf = 0;

    if (nsec > 3600) {
	buf = str_printf ("%uh", (unsigned) (nsec / 3600));
	nsec = fmod (nsec, 3600);
    }
    if (nsec > 60) {
	tmp = str_printf ("%um", (unsigned) (nsec / 60));
	nsec = fmod (nsec, 60);
	buf = xstrcat (buf, tmp);
	free (tmp);
    }
    tmp = str_printf ("%.2fs", nsec);
    if (!strcmp (tmp + strlen(tmp) - 4, ".00s"))
	strcpy (tmp + strlen(tmp) - 4, "s");
    buf = xstrcat (buf, tmp);
    free (tmp);

    return (buf);
}

static char *bytestr (double nbytes) {
    static char *buf;

    free (buf);
    
    if (nbytes > (double)1024*1024*1024*1024)
	buf = str_printf ("%.2fTb",
			  nbytes / ((double) 1024*1024*1024*1024));
    else if (nbytes > 1024*1024*1024)
	buf = str_printf ("%.2fGb",
			  nbytes / ((double) 1024*1024*1024));
    else if (nbytes > 1024*1024)
	buf = str_printf ("%.2fMb",
			  nbytes / ((double) 1024*1024));
    else if (nbytes > 1024)
	buf = str_printf ("%.2fKb",
			  nbytes / (double) 1024);
    else
	buf = str_printf ("%gb", nbytes);

    return (buf);
}

int cmd_status (int ac, char **av) {
    int i, j, services_shown = 0;
    unsigned sec_ago;
    int state_count[NR_OF_STATES];

    if (xml_status) {
	printf ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	if (xml_xslt)
	    printf ("<?xml-stylesheet type=\"text/xsl\" "
		    "href=\"%s\"?>\n", xml_xslt);
	printf ("<status>\n");
    }

    for (i = 0; i < nservice; i++) {
	if (ac > 1 && strcmp (service[i].name, av[0]))
	    continue;
	if (services_shown++) {
	    if (xml_status)
		printf (" </service>\n");
	    else
		putchar ('\n');
	}
	alloc_reporter (service + i, 0);
	msg ("Reporting service %s (pid = %d)",
	     service[i].name, servicereport->pid);

	/* Count the back ends by their state. */
	for (j = 0; j < NR_OF_STATES; j++)
	    state_count[j] = 0;
	for (j = 0; j < service[i].nbackend; j++)
	    state_count[servicereport->backendstate[j].actual_avail]++;
	
	if (tabular_status)
	    printf ("%s ", service[i].name);
	else if (xml_status) {
	    printf (" <service id=\"%d\" name=\"%s\">\n"
		    "  <connections>%d</connections>\n"
		    "  <lastbackend>%d</lastbackend>\n"
		    "  <pid>%u</pid>\n",
		    i, service[i].name, servicereport->nclients,
		    servicereport->last_backend,
		    servicereport->pid);
	    for (j = 0; j < NR_OF_STATES; j++)
		printf ("  <backends_%s>%d</backends_%s>\n",
			state_to_string ( (Backendavail)j ),
			state_count[j],
			state_to_string ( (Backendavail)j ));
	} else 	
	    printf ("Service      : %s, %d live connections, "
		    "last backend %d, at pid %u\n",
		    service[i].name, servicereport->nclients,
		    servicereport->last_backend, servicereport->pid);
	for (j = 0; j < service[i].nbackend; j++) {
	    if (ac > 2 && strcmp (service[i].backend[j].name, av[1]))
		continue;
	    if (! servicereport->backendstate[j].laststamp)
		sec_ago = 0;
	    else
		sec_ago = time(0) - servicereport->backendstate[j].laststamp;
	    if (tabular_status)
		printf ("%s=%s ", service[i].backend[j].name,
		  state_to_string(servicereport->backendstate[j].actual_avail));
	    else if (xml_status) {
		printf ("  <backend id=\"%d\" name=\"%s\">\n"
			"   <availability id=\"%d\">%s</availability>\n"
			"   <clients>%u</clients>\n"
			"   <failures>%lu</failures>\n"
			"   <connections>%lu</connections>\n"
			"   <duration sec=\"%g\">%s</duration>\n"
			"   <throughput bytes=\"%g\">%s</throughput>\n",
			j, service[i].backend[j].name,
			servicereport->backendstate[j].actual_avail,
			state_to_string(
			    servicereport->backendstate[j].actual_avail),
			servicereport->backendstate[j].nclients,
			servicereport->backendstate[j].failures,
			servicereport->backendstate[j].totuses,
			servicereport->backendstate[j].nsec,
			timestr (servicereport->backendstate[j].nsec),
			servicereport->backendstate[j].nbytes,
			bytestr (servicereport->backendstate[j].nbytes));
		printf ("   <lastaccess sec=\"%u\">%s</lastaccess>\n"
			"   <server>%s</server>\n"
			"   <port>%d</port>\n",
			sec_ago, timestr( (double) sec_ago),
			servicereport->backendstate[j].actual_server,
			servicereport->backendstate[j].actual_port);
		if (service[i].type == type_http)
		    printf ("   <sessions>%lu</sessions>\n",
			    servicereport->backendstate[j].sessions);
		printf ("  </backend>\n");
	    }
	    else {
		printf ("  Backend %2d : %s is %s, %u live connections\n"
			"    Stats    : %lu failures out of %lu connections",
			j,
			service[i].backend[j].name,
			state_to_string (
			    servicereport->backendstate[j].actual_avail),
			servicereport->backendstate[j].nclients,
			servicereport->backendstate[j].failures,
			servicereport->backendstate[j].totuses);
		if (service[i].type == type_http)
		    printf (", %lu sessions,",
			    servicereport->backendstate[j].sessions);
		printf ("\n"
			"               usage %s, %s",
			timestr (servicereport->backendstate[j].nsec),
			bytestr (servicereport->backendstate[j].nbytes));
		if (service[i].dispatchover)
		    printf (", avg %s, %s",
			  timestr(servicereport->backendstate[j].avg_nsec),
			  bytestr(servicereport->backendstate[j].avg_nbytes));
		printf (", last hit %s ago\n"
			"               address %s:%d\n",
			timestr ( (double) sec_ago),
			servicereport->backendstate[j].actual_server,
			servicereport->backendstate[j].actual_port);
	    }
	}
	
	if (shmdt (servicereport) < 0)
	    error ("Failure releasing reporter memory: %s",
		   strerror(errno));
    }
    if (tabular_status)
	putchar ('\n');
    else if (xml_status) {
	if (services_shown)
	    printf (" </service>\n");
	printf ("</status>\n");
    }

    return (1);
}
