/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

int cmd_tell (int ac, char **av) {
    Service *target_service = 0;
    int target_backend = -1, i;
    Backendavail avail;
    int newstate, port;
    char *hostname, *cp;

    /* Locate service */
    for (i = 0; i < nservice; i++)
	if (!strcasecmp (service[i].name, av[0])) {
	    target_service = service + i;
	    break;
	}
    if (!target_service)
	error ("Service '%s' isn't known. "
	       "Maybe the configuration has changed?", av[0]);

    /* Locate back end */
    for (i = 0; i < target_service->nbackend; i++)
	if (!strcasecmp (target_service->backend[i].name, av[1])) {
	    target_backend = i;
	    break;
	}
    if (target_backend == -1)
	error ("Backend '%s' of service '%s' isn't known. "
	       "Maybe the configuration has changed?", av[1], av[0]);

    /* Let's see what we need to tell the back end of that service. */
    if (!strcmp (av[2], "state")) {
	/* Locate new state: either the direct number, or alpha name */
	if (sscanf (av[3], "%d", &newstate) > 0) {
	    avail = (Backendavail) newstate;
	    if (avail > st_unknown)
		error ("Bad state number '%s'", av[3]);
	} else if ( (avail = string_to_state (av[3])) == st_unknown )
	    error ("Incorrect state '%s'.", av[3]);

	/* Got all data. Poke the new state into shared memory. */
	alloc_reporter (target_service, 0);
	servicereport->backendstate[target_backend].actual_avail = avail;
	servicereport->backendstate[target_backend].fail  = 0;
	msg ("Marked backend %s of service %s as %s.",
	     av[1], av[0], state_to_string (avail));
    } else if (!strcmp (av[2], "server")) {
	/* Get the hostname and port part. */
	hostname = av[3];
	if (! (cp = strchr (hostname, ':')) )
	    error ("Bad address '%s' (hostname:port expected)", av[3]);
	*cp = 0;
	if (sscanf (cp + 1, "%d", &port) < 1)
	    error ("Bad port part in address");	
	/* Got all data. Poke the new state into shared memory. */
	alloc_reporter (target_service, 0);
	strncpy (servicereport->backendstate[target_backend].actual_server,
		 hostname, FQDN_LENGTH - 1);
	servicereport->backendstate[target_backend].actual_port = port;
	msg ("Address for backend %s of service %s set to %s:%d.",
	     av[1], av[0], hostname, port);
    } else {
	usage();
    }
    /* All done.. success */
    return (1);
}
	       
