/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

#define MSGSTR(x,y) msg(x ": [%s]", y ? y : "<none>")
#define MSGUNS(x,y) msg(x ": [%u]", y)
#define MSGINT(x,y) msg(x ": [%d]", y)

#define BMSGSTR(x,y,z) msg("Backend %d: " y " [%s]", x + 1, z ? z : "<none>")
#define BMSGINT(x,y,z) msg("Backend %d: " y " [%d]", x + 1, z)
#define BMSGUNS(x,y,z) msg("Backend %d: " y " [%u]", x + 1, z)

static char *dispatchstr (Dispatchtype t) {
    switch (t) {
    case ds_roundrobin:
	return ("roundrobin");
    case ds_random:
	return ("random");
    case ds_bysize:
	return ("bysize");
    case ds_byduration:
	return ("byduration");
    case ds_byorder:
	return ("byorder");
    case ds_byconnections:
	return ("byconnections");
    case ds_byclientip:
	return ("byclientip");
    case ds_externalhandler:
	return ("externalhandler");
    default:
	return ("*** cannot display ***");
    }
}

static char *servicestr (Servicetype t) {
    switch (t) {
    case type_any:
	return ("any");
    case type_http:
	return ("http");
    default:
	return ("*** cannot display ***");
    }
}

static char *inspectionstr (HttpInspection t) {
    switch (t) {
    case ins_deep:
	return ("deep");
    case ins_shallow:
	return ("shallow");
    default:
	return ("*** cannot display ***");
    }
}

static void headers (int nr, char **hdr, int nhdr, char *desc) {
    int i;

    msg ("Backend %d: %d %s header(s)", nr + 1, nhdr, desc);
    for (i = 0; i < nhdr; i++)
	msg ("Backend %d: %s #%d: %s", nr + 1, desc, i + 1, hdr[i]);
}

void config_msg () {
    int i;
    
    if (!activeservice->verbosity)
	return;

    msg ("*** Overview of initial configuration of service '%s' ***",
	 activeservice->name);
    MSGSTR ("Bind address", activeservice->bind);
    MSGUNS ("Bind port", activeservice->port);
    MSGUNS ("Verbosity", activeservice->verbosity);
    MSGSTR ("Dispatch type", dispatchstr (activeservice->dispatchtype));
    MSGSTR ("External handler", activeservice->dispatchext);
    MSGUNS ("Reviving interval", activeservice->rev_interval);
    MSGUNS ("Check interval", activeservice->check_interval);
    MSGSTR ("External checker", activeservice->check_cmd);
    MSGUNS ("TCP backlog", activeservice->backlog);
    MSGUNS ("SHM key", activeservice->shmkey);
    MSGUNS ("Connection timeout", activeservice->connectiontimeout);
    MSGUNS ("Max connections", activeservice->maxconnections);
    MSGSTR ("Service type", servicestr (activeservice->type));
    MSGSTR ("HTTP inspection", inspectionstr (activeservice->inspection));
    MSGSTR ("Allow file", activeservice->allowfile);
    MSGSTR ("Deny file", activeservice->denyfile);
    MSGINT ("UID", activeservice->uid);
    MSGINT ("GID", activeservice->gid);
    MSGINT ("Initial deny chain", activeservice->ndenychain);
    for (i = 0; i < activeservice->ndenychain; i++)
	msg ("Deny #%d: %4.4x/%4.4x", i + 1, activeservice->denychain[i].ip,
	     activeservice->denychain[i].mask);
    MSGINT ("Initial allow chain", activeservice->nallowchain);
    for (i = 0; i < activeservice->nallowchain; i++)
	msg ("Allow #%d: %4.4x/%4.4x", i + 1, activeservice->allowchain[i].ip,
	     activeservice->allowchain[i].mask);
    MSGINT ("Back ends", activeservice->nbackend);

    for (i = 0; i < activeservice->nbackend; i++) {
	BMSGSTR (i, "name", activeservice->backend[i].name);
	BMSGINT (i, "port", activeservice->backend[i].initial_port);
	BMSGSTR (i, "server", activeservice->backend[i].initial_server);
	BMSGINT (i, "verbosity", activeservice->backend[i].verbosity);
	BMSGINT (i, "weight", activeservice->backend[i].weight);
	BMSGINT (i, "decay", activeservice->backend[i].decay);
	BMSGSTR (i, "onstart cmd", activeservice->backend[i].onstart);
	BMSGSTR (i, "onfail cmd", activeservice->backend[i].onfail);
	BMSGSTR (i, "onend cmd", activeservice->backend[i].onend);
	BMSGSTR (i, "dumplog", activeservice->backend[i].dumpfile);
	BMSGSTR (i, "thruputlog", activeservice->backend[i].thruputfile);
	BMSGSTR (i, "timinglog", activeservice->backend[i].timinglog);
	BMSGUNS (i, "maxconnections", activeservice->backend[i].maxconnections);
	BMSGSTR (i, "stickycookie", activeservice->backend[i].stickycookie);
	BMSGINT (i, "retries", activeservice->backend[i].retries);
	headers (i, activeservice->backend[i].addclientheader,
		 activeservice->backend[i].naddclientheader,
		 "addclientheader");
	headers (i, activeservice->backend[i].setclientheader,
		 activeservice->backend[i].nsetclientheader,
		 "setclientheader");
	headers (i, activeservice->backend[i].appendclientheader,
		 activeservice->backend[i].nappendclientheader,
		 "appendclientheader");
	headers (i, activeservice->backend[i].addserverheader,
		 activeservice->backend[i].naddserverheader,
		 "addserverheader");
	headers (i, activeservice->backend[i].setserverheader,
		 activeservice->backend[i].nsetserverheader,
		 "setserverheader");
	headers (i, activeservice->backend[i].appendserverheader,
		 activeservice->backend[i].nappendserverheader,
		 "appendserverheader");
	BMSGSTR (i, "availability",
		 state_to_string (activeservice->backend[i].initial_avail));
    }
}
    
