/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

void dealloc_reporter (Service *s) {
    int shmid;

    msg ("Freeing reporting memory for service %s (key %d)",
	 s->name, s->shmkey);

    /* Get the shared memory ID. If this fails and we're in stage 0,
     * then ignore the error. The true handler will have already released it.
     */
    if ( (shmid = shmget (s->shmkey, sizeof(Servicereport),
			  0644) ) >= 0) {
	msg ("Got existing shm at id %d", shmid);
	/* Mark to be released. If this fails then warn. */
	if (shmctl (shmid, IPC_RMID, 0) < 0 && program_stage != stage_main)
	    warning ("WARNING: Cannot mark shared memory as destructable: %s "
		     "Root might need to 'ipcrm -m %d' and "
		     "'ipcrm -s %d' to clean up!",
		     strerror(errno), shmid, semid);
	if (semctl (semid, 0, IPC_RMID) < 0 && program_stage != stage_main)
	    warning ("WARNING: Cannot remove semaphore: %s "
		     "Root might need to 'ipcrm -s %d' to clean up!",
		     strerror(errno), semid);
    }
    else if (program_stage != stage_main) {
	/* Errno 2 means that the shm block was already removed (by a
	 * different listener) Don't warn if that's the case.
	 */
	if (errno != 2)
	    warning ("Cannot get shared memory for service %s (key %d: %d/%s)",
		     s->name, s->shmkey, errno, strerror(errno));
    }
}
