/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* For more debugging information, undefine the following and set
 * "verbosity on" in the configuration file. */
/* #define DEBUG */

#ifdef DEBUG

static void debughdr (HttpHeader *h) {
    int i;
    msg ("http_header_read: %d headers so far", h->nheader);
    for (i = 0; i < h->nheader; i++)
	msg ("http header read: header[%d] '%s'",
	     i, h->header[i]);
}

#define SHOWHEADERS(h)  debughdr((h))
#define MSG(m)		m;

#else

#define SHOWHEADERS(h)
#define MSG(m)

#endif

void http_header_read (HttpHeader *h, int sock, CopyDirection dir) {
    unsigned char *hdrbuf = 0, *start, *buf, *cp;
    int hdrbuflen = 0, usedlen, *bufpos, *bufmax, i;

    msg ("reading HTTP headers from %s",
	 dir == dir_client_to_server ? "client" : "server");

    /* Read from network until we match [\r]\n[\r]\n */
    if (dir == dir_client_to_server) {
	buf = clbuf;
	bufpos = &clbufpos;
	bufmax = &clbufmax;
    } else {
	buf = srbuf;
	bufpos = &srbufpos;
	bufmax = &srbufmax;
    }
    while (1) {
	if (! *bufmax) {
	    *bufmax = net_read (sock, opt.tcp_bufsz, buf,
				dir == dir_client_to_server);
	    MSG(msg("Got %d network bytes (%s)", *bufmax, buf));
	}
	hdrbuf = xrealloc (hdrbuf, hdrbuflen + *bufmax);
	memcpy (hdrbuf + hdrbuflen, buf + *bufpos, *bufmax);
	hdrbuflen += *bufmax;
	MSG(msg("Headers buffer so far: %d bytes (%s)", hdrbuflen, hdrbuf));

	*bufmax = 0;

	/* Try to find header ending sequence */
	if ( (cp = strnstr ((char const *)hdrbuf, "\r\n\r\n",
			    (unsigned)hdrbuflen)) ||
	     (cp = strnstr ((char const *)hdrbuf, "\n\n",
			    (unsigned)hdrbuflen)) ) {
	    break;
	}
    }

    MSG(msg("Headers done, end at %s", cp));
	
    /* Skip over the ending sequence and save remainder of buffer
     * for next usage */
    for (i = 0; i < 2; i++) {
	if (*cp == '\r')
	    cp++;
	if (*cp == '\n')
	    cp++;
    }
    usedlen = cp - hdrbuf;
    if (usedlen < hdrbuflen) {
	/* NOTE: We can always poke back the remainder, this is by definition
	 * not longer than opt.tcp_bufsize */
	memcpy (buf, cp, hdrbuflen - usedlen);
	*bufpos = 0;
	*bufmax = hdrbuflen - usedlen;
	MSG(msg("Pushed back %d bytes (%s)", *bufmax, buf));
    } else {
	*bufmax = 0;
	MSG(msg("No remainding network for next read"));
    }
	
    /* Split up the header buffer into separate headers and store them */
    start = hdrbuf;
    while (1) {
	if (! (cp = memchr (start, '\r', usedlen - (start - hdrbuf))) )
	    cp = memchr (start, '\n', usedlen - (start - hdrbuf));
	if (!cp)
	    break;
	MSG(msg("Header %d: (%d bytes) at %s", h->nheader + 1,
		cp - start, start));
	if (cp - start) {
	    h->header = xrealloc (h->header, (h->nheader + 1) * sizeof(char*));
	    h->header[h->nheader] = xmalloc (cp - start + 1);
	    memcpy (h->header[h->nheader], start, cp - start);
	    h->header[h->nheader][cp - start] = 0;
	    h->nheader++;
	}
	start = cp + 1;
	if (*start == '\n')
	    start++;
    }
    
    free (hdrbuf);

    SHOWHEADERS(h);
}
