/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

void http_log_activity (double start, double now, char const *fmt, ...) {
    FILE *f;
    va_list args;

    if (activeservice->type != type_http || program_stage == stage_retrying ||
	current_backend < 0 ||
	!activeservice->backend[current_backend].timinglog ||
	! *activeservice->backend[current_backend].timinglog)
	return;
    
    if (!now)
	now = timeofday();
    if (! (f = fopen (activeservice->backend[current_backend].timinglog,
		      "a")) &&
	! (f = fopen (activeservice->backend[current_backend].timinglog,
		      "w")) ) {
	warning ("cannot write HTTP timings log %s: %s",
		 activeservice->backend[current_backend].timinglog,
		 strerror(errno));
	return;
    }

#if HAVE_FLOCK == 1
    flock (fileno(f), LOCK_EX);
#elif HAVE_LOCKF == 1
    lockf (fileno(f), F_LOCK, 0);
#endif

    fprintf (f, "%7.7d %15f ", getpid(), now - start);
    va_start (args, fmt);
    vfprintf (f, fmt, args);
    fputc ('\n', f);
    
#if defined HAVE_FLOCK    
    flock (fileno(f), LOCK_UN);
#elif defined HAVE_LOCKF
    lockf (fileno(f), F_ULOCK, 0);
#endif

    fclose (f);
}
