/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

int http_serversocket (HttpHeader *m, int *is_continuation) {
    int i, sock;

    msg ("searching for back end for HTTP request");

    /* Try to find a sticky cookie in the request. */
    for (i = 0; i < activeservice->nbackend; i++) {
	if (servicereport->backendstate[i].actual_avail == st_unavailable)
	    continue;
	if (activeservice->backend[i].maxconnections > 0 &&
	    activeservice->backend[i].maxconnections <=
	    servicereport->backendstate[i].nclients)
	    continue;
	if (http_header_hascookie (m,
				   activeservice->backend[i].stickycookie)) {
	    msg ("HTTP backend %d selected due to cookie '%s'",
		 i, 
		 activeservice->backend[i].stickycookie);

	    /* Got the target back end. Try to connect to it.
	     * If it doesn't succeed, then we'll do a failover to
	     * any other back end. */
	    current_backend = i;
	    if ( (sock = backend_connect ()) >= 0 ) {
		*is_continuation = 1;
		set_program_title ("crossroads - Service %s: "
				   "serving %s to %s",
				 activeservice->name,
				 client_ip,
				 activeservice->backend[current_backend].name);
		return (sock);
	    }	
	}
    }

    /* We got here because this is the first time 'round or because
     * a previously established back end went dead in the middle of a
     * session. So: Loop 'till we find a back end, or until we fail. */
    while (1) {
	/* No back end? Nogo. */
	if (! backend_count()) {
	    msg ("out of back ends while scanning for HTTP back end");
	    return (-1);
	}

	/* Try one. */
	choose_backend();
	if (current_backend < 0) {
	    warning ("No back end for request.");
	    return (-2);
	}

	msg ("trying back end %s for new HTTP connection",
	     activeservice->backend[current_backend].name);
	if ( (sock = backend_connect ()) >= 0 ) {
	    servicereport->backendstate[current_backend].sessions++;
	    *is_continuation = 0;
	    set_program_title ("crossroads - Service %s: serving %s to %s",
			       activeservice->name,
			       client_ip,
			       activeservice->backend[current_backend].name);
	    return (sock);
	}
    }
}
