/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* Signal handler. NOTE - don't use non-reentrant functions in code
 * that doesn't exit! */
void interrupt (int sig) {

    switch (sig) {

    case SIGHUP:
	/* Reload allow/deny files upon next loop */
	reload_allow_deny++;
	break;

    case SIGPIPE:
	/* SIGPIPE shouldn't happen, the sockets are set up not to
	 * generate this. Incase it does: ignore, the next read/write
	 * will stop. */
	break;

    default:
	interrupted++;
	
	switch (program_stage) {
	case stage_waiting:
	    warning ("listener caught signal %d, closing socket %d and exiting",
		     sig, listen_sock);
	    shutdown (listen_sock, SHUT_RDWR);
	    close (listen_sock);
	    exit (0);
	    break;
	    
	case stage_serving:
	    /* This will serve the current connection and then stop. */
	    break;
	    
	case stage_retrying:
	    warning ("wakeup handler caught signal %d, "
		     "stopping wake ups and exiting", sig);
	    exit (0);

	default:
	    msg ("Caught signal %d in program stage %s, exiting", sig,
		 stage_to_string (program_stage));
	    exit (0);
	}	
    }
}
