/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

int ipf_loadfile (char const *fname, IpFilter **chain, int *nchain) {
    FILE *f;
    char buf[80];
    int nloaded = 0, ret = 0;
    IpFilter filter;

    /* No file? No update. */
    if (!fname || !*fname)
	return (0);

    msg ("loading IP filter file: '%s'", fname);

    if (! (f = fopen (fname, "r")) ) {
	warning ("can't read filter file '%s': %s", fname, strerror(errno));
	return (0);
    }

    free (*chain);
    *chain = 0;
    *nchain = 0;

    while (1) {
	if (fscanf (f, " %79s ", buf) < 1)
	    break;
	if (ipf_parse (buf, &filter)) {
	    warning ("bad ip filter in file '%s'", fname);
	    ret++;
	} else {
	    msg ("got ip filter specifier '%s'", buf);
	    *chain = xrealloc (*chain, (*nchain + 1) * sizeof(IpFilter));
	    (*chain)[*nchain] = filter;
	    (*nchain)++;
	    nloaded++;
	}
    }

    msg ("loaded IP filter file '%s': %d successfully loaded",
	 fname, nloaded);

    fclose (f);
    return (ret);
}
