%{
#include "../crossroads.h"
#include "parser.h"

/* Redefinition of YY_INPUT to respect the parserstate. */
#define YY_INPUT(buf,res,max)  (res) = parser_input( (buf), (max) )
#define yywrap parser_close

/* Globals */
static int i;

/* Lexer debugging related */
static void lmsg (char const *fmt, ...) {
    va_list args;

    if (parser_debug) {
	va_start (args, fmt);
	printf ("LEXER:  ");
	vprintf (fmt, args);
    }
}
%}

%pointer
%x stringstate instr commentstate includestate definestate defineredef definesym

%%

^#[ \t]*include {
    lmsg ("include starts\n");
    BEGIN (includestate);
}


^#[ \t]*define {
    lmsg ("define starts\n");
    BEGIN (definestate);
}

\/\/.*\n {
    lmsg ("comment\n");
    yylineno++;
}

options {
    lmsg ("options\n");
    return (OPTIONS);
}

logactivity {
    lmsg ("logactivity\n");
    return (LOGACTIVITY);
}

tcpbuffersize {
    lmsg ("tcpbuffersize\n");
    return (TCPBUFFERSIZE);
}

dnscachettl {
    lmsg ("dnscachettl\n");
    return (DNSCACHETTL);
}

logfacility {
    lmsg ("logfacility\n");
    return (LOGFACILITY);
}

shmpermissions {
    lmsg ("shmpermissions\n");
    return (SHMPERMISSIONS);
}

sloppyportbind {
    lmsg ("sloppyportbind\n");
    return (SLOPPYPORTBIND);
}

leaveprocesstitle {
    lmsg ("leaveprocesstitle\n");
    return (LEAVEPROCESSTITLE);
}

service {
    lmsg ("service\n");
    return (SERVICE);
}

bindto {
    lmsg ("bindto\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (BINDTO);
}

port {
    lmsg ("port\n");
    return (PORT);
}

over {
    lmsg ("over\n");
    return (OVER);
}

shmkey {
    lmsg ("shmkey\n");
    return (SHMKEY);
}

backend {
    lmsg ("backend\n");
    return (BACKEND);
}

state {
    lmsg ("state\n");
    return (STATE);
}

(available)|(up) {
    lmsg ("available\n");
    return (AVAILABLE);
}

unavailable {
    lmsg ("unavailable\n");
    return (UNAVAILABLE);
}

down {
    lmsg ("down\n");
    return (DOWN);
}

backlog {
    lmsg ("backlog\n");
    return (BACKLOG);
}

retries {
    lmsg ("retries\n");
    return (RETRIES);
}

(verbosity)|(verbose) {
    lmsg ("verbosity\n");
    return (VERBOSITY);
}

connectiontimeout {
    lmsg ("connectiontimeout\n");
    return (CONNECTIONTIMEOUT);
}

maxconnections {
    lmsg ("maxconnections\n");
    return (MAXCONNECTIONS);
}

weight {
    lmsg ("weight\n");
    return (WEIGHT);
}

decay {
    lmsg ("decay\n");
    return (DECAY);
}

server {
    lmsg ("server\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (SERVER);
}

dispatchmode {
    lmsg ("dispatchmode\n");
    return (DISPATCHMODE);
}

roundrobin {
    lmsg ("roundrobin\n");
    return (ROUNDROBIN);
}

random {
    lmsg ("random\n");
    return (RANDOM);
}

byduration {
    lmsg ("byduration\n");
    return (BYDURATION);
}

bysize {
    lmsg ("bysize\n");
    return (BYSIZE);
}

byorder	{
    lmsg ("byorder\n");
    return (BYORDER);
}

byconnections {
    lmsg ("byconnections\n");
    return (BYCONNECTIONS);
}

externalhandler {
    lmsg ("externalhandler\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (EXTERNALHANDLER);
}

byclientip {
    lmsg ("byclientip\n");
    return (BYCLIENTIP);
}    

useraccount {
    lmsg ("useraccount\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (USERACCOUNT);
}

revivinginterval {
    lmsg ("revivinginterval\n");
    return (REVIVINGINTERVAL);
}

checkinterval {
    lmsg ("checkinterval\n");
    return (CHECKINTERVAL);
}

type {
    lmsg ("type\n");
    return (TYPE);
}

headerinspection {
    lmsg ("headerinspection\n");
    return (HEADERINSPECTION);
}

deep {
    lmsg ("deep\n");
    return (DEEP);
}

shallow {
    lmsg ("shallow\n");
    return (SHALLOW);
}

any {
    lmsg ("any\n");
    return (ANY);
}

stickyhttp {
    lmsg ("stickyhttp\n");
    warning ("The 'stickyhttp protocol is obsolte.\n"
	     "You should change to 'http'.");
    return (HTTP);
}

http {
    lmsg ("http\n");
    return (HTTP);
}

throughputlog {
    lmsg ("throughputlog\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (THROUGHPUTLOG);
}

trafficlog {
    lmsg ("trafficlog\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (TRAFFICLOG);
}

httptiminglog {
    lmsg ("httptiminglog\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (HTTPTIMINGLOG);
}

dumptraffic {
    lmsg ("dumptraffic\n");
    warning ("The 'dumptraffic' statement is obsolete.\n"
	     "You should change to 'trafficlog'.");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (TRAFFICLOG);
}

onstart {
    lmsg ("onstart\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ONSTART);
}

onfail {
    lmsg ("onfail\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ONFAIL);
}

onend {
    lmsg ("onend\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ONEND);
}

stickycookie {
    lmsg ("stickycookie\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (STICKYCOOKIE);
}

addclientheader {
    lmsg ("addclientheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ADDCLIENTHEADER);
}

setclientheader {
    lmsg ("setclientheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (SETCLIENTHEADER);
}

appendclientheader {
    lmsg ("appendclientheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (APPENDCLIENTHEADER);
}

addserverheader {
    lmsg ("addserverheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ADDSERVERHEADER);
}

setserverheader {
    lmsg ("setserverheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (SETSERVERHEADER);
}

appendserverheader {
    lmsg ("appendserverheader\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (APPENDSERVERHEADER);
}

allowfrom {
    lmsg ("allowfrom\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ALLOWFROM);
}

denyfrom {
    lmsg ("denyfrom\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (DENYFROM);
}

allowfile {
    lmsg ("allowfile\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (ALLOWFILE);
}

denyfile {
    lmsg ("denyfile\n");
    BEGIN (stringstate);
    free (laststring);
    laststring = 0;
    return (DENYFILE);
}

on|true|yes {
    lmsg ("on\n");
    return (ON);
}

off|false|no {
    lmsg ("off\n");
    return (OFF);
}

[a-zA-Z_][a-zA-Z0-9_]* {
    lmsg ("identifier [%s]\n", yytext);
    return (IDENTIFIER);
}

[0-9]+ {
    lmsg ("number [%s]\n", yytext);
    return (NUMBER);
}

[ \t]+ {
    lmsg ("space(s)\n");
}

\n {
    lmsg ("newline\n");
    yylineno++;
}

. {
    lmsg ("lone char ['%c' (ASCII %d)]\n",
	  isprint(*yytext) ? *yytext : '.',
	  *yytext);
    return (*yytext);
}

\/\* {
    lmsg ("C-comment starts\n");
    BEGIN(commentstate);
}
<commentstate>\*\/ {
    lmsg ("C-comment ends\n");
    BEGIN(0);
}
<commentstate>\n {
    yylineno++;
}
<commentstate>.	;

<stringstate>[ \r\n\t] {
    lmsg ("string state: skipping initial spaces [ASCII %d]\n", *yytext);
    if (*yytext == '\n')
	yylineno++;
}
<stringstate>. {
    lmsg ("string state: entering instring on [%s (ASCII %d)]\n",
	  yytext, *yytext);
    unput (*yytext);
    BEGIN (instr);
}

<instr>(\\\")|(\\\') {
    lmsg ("string part [%s]\n", yytext);
    laststring = xstrcat (laststring, yytext);
}

<instr>\"[^\"]*\"	{
    lmsg ("string part in double quotes [%s]\n", yytext);
    laststring = xstrcat (laststring, yytext + 1);
    laststring[strlen(laststring) - 1] = 0;
}
<instr>\'[^\']*\' {
    lmsg ("string part in single quotes [%s]\n", yytext);
    laststring = xstrcat (laststring, yytext + 1);
    laststring[strlen(laststring) - 1] = 0;
}
<instr>; {
    BEGIN (0);
    /* Strip terminating whitespace incase the ; is preceded by them */
    if (laststring && *laststring)
	for (i = strlen(laststring) - 1; i >= 0 && isspace(laststring[i]); i--)
	    laststring[i] = 0;
    unput (';');
    lmsg ("string complete [%s]\n", laststring);
    return (STRING);
}
<instr>" " {
    if (laststring) {
	laststring = xstrcat (laststring, yytext);
	lmsg ("string part space [%s]\n", yytext);
    }
}
<instr>. {
    lmsg ("string part char [%s]\n", yytext);
    laststring = xstrcat (laststring, yytext);
}
<instr>\n	{
    if (laststring) {
	laststring = xstrcat (laststring, " ");
	lmsg ("string part: newline, now space\n");
    }
    yylineno++;
}

<includestate>[<>\"a-zA-Z0-9_\-\./:\\]+ {
    lmsg ("includefile\n");
    /* parser_skipline(); */
    parser_open (yytext);
    BEGIN (0);
}
<includestate><<EOF>> {
    error ("Unterminated #include");
}
<includestate>[ \t] ;
<includestate>\n {
    yylineno++;
}
<includestate>. {
    error ("Illegal character 0x%2.2x in #include line", *yytext);
}

<definestate>[ \s]+ ;
<definestate>[A-Za-z_][A-Za-z0-9_]* {
    symtab_start (yytext);
    BEGIN (defineredef);
}
<definestate,defineredef,definesym><<EOF>> {
    error ("Unterminated #define");
}
<definestate,defineredef>\n {
    error ("Unterminated #define");
}
<definestate>. {
    error ("Unrecognized character '%c' in #define", *yytext);
}

<defineredef>[ \t]* {
    BEGIN (definesym);
}
<definesym>.* {
    symtab_set (yytext);
}
<definesym>\n {
    yylineno++;
    symtab_end();
    BEGIN (0);
}
