/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

typedef struct {
    int type;
    char *desc;
} Rlimits;

static Rlimits rlimits[] = {
    { RLIMIT_CORE,		"core file size" },
    { RLIMIT_CPU,		"max seconds CPU usage" },
    { RLIMIT_DATA,		"max bytes data segment usage" },
    { RLIMIT_FSIZE,		"max bytes in created files" },
    #ifdef RLIMIT_MEMLOCK
    { RLIMIT_MEMLOCK,		"max bytes in locked memory" },
    #endif
    { RLIMIT_NOFILE,		"max nr of open files" },
    #ifdef RLIMIT_NPROC
    { RLIMIT_NPROC,		"max nr of processes of this id" },
    #endif
    #ifdef RLIMIT_RSS
    { RLIMIT_RSS,		"max bytes in resident memory" },
    #endif
    { RLIMIT_STACK,		"max bytes in stack" },
};

void limits_msg () {
    struct rlimit lim;
    unsigned i;
    
    if (!activeservice->verbosity)
	return;

    msg ("*** Overview of run limits ***");
    for (i = 0; i < sizeof(rlimits) / sizeof(Rlimits); i++) {
	if (getrlimit (rlimits[i].type, &lim))
	    error ("failed to get limits for '%s'", rlimits[i].desc);
	msg ("Limit for '%s': current %u, max %u",
	     rlimits[i].desc,
	     (unsigned) lim.rlim_cur, (unsigned)lim.rlim_max);
    }
}
