/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
    
#include "../crossroads.h"

int make_socket (int port, char const *ipaddr) {
    int sock, val = 1;
    struct sockaddr_in name;

    /* Create the socket. */
    if ( (sock = socket (PF_INET, SOCK_STREAM, 0)) < 0) {       
	warning ("cannot create network socket: %s", strerror(errno));
	return (-1);
    }
    if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
	warning ("cannot set socket options (reuseaddr): %s", strerror(errno));
	close (sock);
	return (-2);
    }
#ifdef SO_NOSIGPIPE    
    if (setsockopt (sock, SOL_SOCKET, SO_NOSIGPIPE, &val, sizeof(val))) {
	warning ("cannot set socket options (nosigpipe): %s", strerror(errno));
	close (sock);
	return (-2);
    }
#endif
    
    /* Give the socket a name. */
    name.sin_family = AF_INET;
    name.sin_port = htons (port);

    /* Propagate 'bindto' if given. */
    if (ipaddr && *ipaddr && strcmp(ipaddr, "any")) {
	msg ("creating socket for IP address '%s'", ipaddr);
	if ( (name.sin_addr.s_addr = inet_addr (ipaddr)) == INADDR_NONE ) {
	    close (sock);
	    error ("cannot convert address '%s' to network bytes", ipaddr);
	}
	/*
	if (!inet_aton (ipaddr, &name.sin_addr.s_addr)) {
	    close (sock);
	    error ("Cannnot assign bindto address '%s'", ipaddr);
	}
	*/
    } else {
	msg ("creating socket for all interfaces");
	name.sin_addr.s_addr = htonl (INADDR_ANY);
    }

    /* Finally, bind the socket. */
    if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0) {
	warning ("Cannot bind socket to port %d: %s",
		 port, strerror(errno));
	close (sock);
	return (-3);
    }

    return (sock);
}
