/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

unsigned char *net_bufread (int sock, int max, int *nread,
			    int is_client) {
    unsigned char *buf, *ret;
    int *bufpos, *bufmax, rest;

    /* Make sure we have the buffers. */
    net_allocbufs();

    /* Set our own ptrs */
    if (is_client) {
	buf = clbuf;
	bufpos = &clbufpos;
	bufmax = &clbufmax;
    } else {
	buf = srbuf;
	bufpos = &srbufpos;
	bufmax = &srbufmax;
    }

    /* If buffer is empty, do an actual read */
    if (! *bufmax) {
	*bufmax = net_read (sock, opt.tcp_bufsz, buf, is_client);
	msg ("Got %d bytes from %s", *bufmax, is_client ? "client" : "server");
	*bufpos = 0;
    }

    /* Got buffer contents. How much do we return? */
    rest = *bufmax - *bufpos;
    
    if (rest > max) {
	/* Caller wants less than what we have in cache. */
	if (nread)
	    *nread = max;
	ret = buf + *bufpos;
	*bufpos += max;
    } else {
	/* Caller wants more than what we have in cache.
	 * We'll return what we got. */
	if (nread)
	    *nread = rest;
	ret = buf + *bufpos;
	*bufmax = *bufpos = 0;
    }
    
    return (ret);
}
