/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

int net_copy (int cl, int sr, int max, unsigned char *buf) {
    fd_set readset, exset;
    struct timeval tv, *tvp, tv1, tv2;
    int nfd;
    int nread, nwritten, totwritten;
    CopyDirection dir;
    double microsec;

    /* Prepare select() sets */
    FD_ZERO (&readset);
    FD_ZERO (&exset);

    FD_SET (cl, &readset);
    FD_SET (cl, &exset);
    FD_SET (sr, &readset);
    FD_SET (sr, &exset);

    /* Prepare timout state */
    if (activeservice->connectiontimeout) {
	tv.tv_sec = activeservice->connectiontimeout;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    /* Wait for a socket to become readable */
    gettimeofday (&tv1, 0);
    nfd = select (FD_SETSIZE, &readset, 0, &exset, tvp);
    if (nfd < 1) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, st_available);
	error ("network copy stopped after timeout");
    }

    /* Check for exceptions. */
    if (FD_ISSET (cl, &exset)) {
	decr_client_count();
	log_activity_end();
	error ("client exception");
    }
    if (FD_ISSET (sr, &exset)) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, st_unavailable);
	error ("server exception");
    }

    /* Do the read. */
    if (FD_ISSET (sr, &readset))
	dir = dir_server_to_client;
    else
	dir = dir_client_to_server;

    nread = read (dir == dir_client_to_server ? cl : sr, buf, max);
    if (nread < 1) {
	decr_client_count();
	log_activity_end();
	if (nread < 0) {
	    /* An exception on the network socket. If this is the server
	     * that's sending, then we issue an error message.
	     * If this is the client, then we'll just issue a verbose
	     * message; clients are notorious for just dropping dead. */
	    if (dir == dir_server_to_client)
		error ("Read error from server");
	    else {
		msg ("Read error from client");
		exit (0);
	    }
	} else {
	    msg ("%s signals end of data",
		 dir == dir_client_to_server ? "client" : "server");
	    exit (0);
	}
    }

    /* Update thruput / traffic logs. */
    trafficlog (buf, nread, dir);
    thruputlog (buf, nread, dir);

    /* Write to dest. */
    totwritten = 0;
    while (totwritten < nread) {
	nwritten = write (dir == dir_client_to_server ? sr : cl,
			  buf + totwritten,
			  nread - totwritten);
	if (nwritten < 1) {
	    decr_client_count();
	    log_activity_end();
	    warning ("write error when sending data to %s",
		     dir == dir_client_to_server ? "server" : "client");
	    exit (0);
	}
	totwritten += nwritten;
    }

    gettimeofday (&tv2, 0);
    microsec = ( (tv2.tv_sec * 1000000 + tv2.tv_usec) -
                 (tv1.tv_sec * 1000000 + tv1.tv_usec) );

    mark_activity (totwritten, microsec / 1000000, st_intermediate);

    return (nread);
}
