/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* For special-extra verbosity: add "verbose yes" to crossroads.conf
 * and undefine the following line. */
/* #define DEBUG */

#ifdef DEBUG
#define LOGGIT msg ("back end %d", current_backend)
#else
#define LOGGIT
#endif

int net_read (int sock, int max, unsigned char *buf, int is_client) {
    fd_set readset, exset;
    struct timeval tv, *tvp, tv1, tv2;
    int nfd;
    int nread;
    CopyDirection dir;
    double microsec;
    char *desc;

    desc = is_client ? "client" : "server";
    dir  = is_client ? dir_client_to_server : dir_server_to_client;

    msg ("fetching atmost %u bytes from %s", max, desc);

    /* Prepare select() sets */
    FD_ZERO (&readset);
    FD_ZERO (&exset);
    FD_SET (sock, &readset);
    FD_SET (sock, &exset);

    /* Prepare timout state */
    if (activeservice->connectiontimeout) {
	tv.tv_sec = activeservice->connectiontimeout;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    /* Wait for the socket to become readable */
    gettimeofday (&tv1, 0);
    nfd = select (FD_SETSIZE, &readset, 0, &exset, tvp);
    if (nfd < 1) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, is_client ? st_available : st_unavailable);
	error ("read from %s timed out", desc);
    }

    /* Check for exceptions. */
    if (FD_ISSET (sock, &exset)) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, is_client ? st_available : st_unavailable);
	error ("%s exception", desc);
    }

    /* Do the read. */
    nread = read (sock, buf, max);
    LOGGIT;
    if (nread < 1) {
	decr_client_count();
	log_activity_end();
	if (nread < 0) {
	    /* An exception on the network socket. If this is the server
	     * that's sending, then we issue an error message.
	     * If this is the client, then we'll just issue a verbose
	     * message; clients are notorious for just dropping dead. */
	    if (dir == dir_server_to_client)
		error ("Read error from server");
	    else {
		msg ("Read error from client");
		exit (0);
	    }
	} else {
	    msg ("%s signals end of data", desc);
	    exit (0);
	}
    }

    /* Update thruput / traffic logs. */
    trafficlog (buf, nread, dir);
    thruputlog (buf, nread, dir);
    gettimeofday (&tv2, 0);
    microsec = ( (tv2.tv_sec * 1000000 + tv2.tv_usec) -
                 (tv1.tv_sec * 1000000 + tv1.tv_usec) );

    mark_activity (nread, microsec / 1000000, st_intermediate);

    return (nread);
}
