/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

void parser_open (char const *name) {
    FILE *f;
    struct stat statbuf;
    int nread;
    char *actual, *cp;

    /* "name" or <name> is converted to name */
    actual = xstrdup (name);
    cp = actual + strlen(actual) - 1;
    if ( (*actual == '"' && *cp == '"') ||
	 (*actual == '<' && *cp == '>') ) {
	*cp = 0;
	strcpy (actual, actual + 1);
    }

    if (! (f = fopen (actual, "r")) )
	error ("Cannot read configuration %s: %s", actual, strerror(errno));
    if (fstat(fileno(f), &statbuf))
	error ("Cannot stat configuration %s: %s", actual, strerror(errno));

    /* Anticipate the terminating \n, process it and store. */
    yylineno++;
    if (nparserstate > 0) {
	parserstate[nparserstate - 1].lineno = yylineno;
	msg ("%d bytes remain in configuration %s beyond line %d",
	     (int) strlen(parserstate[nparserstate - 1].buf),
	     parserstate[nparserstate - 1].name, yylineno);
    }
    
    parserstate = xrealloc (parserstate,
			    (nparserstate + 1) * sizeof(ParserState));
    parserstate[nparserstate].name = actual;
    parserstate[nparserstate].file = f;
    parserstate[nparserstate].lineno = 0;
    parserstate[nparserstate].buf = xmalloc (statbuf.st_size + 1);
    nread = fread (parserstate[nparserstate].buf, 1, statbuf.st_size, f);
    /* msg ("Got %d bytes from configuration %s", nread, actual); */
    if (nread < statbuf.st_size)
	error ("Read only %d bytes from configuration %s (out of %d)",
	       nread, actual, (int) statbuf.st_size);
    parserstate[nparserstate].buf[statbuf.st_size] = 0;
    nparserstate++;

    yylineno = 0;
    yyin = f;
}


