/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

void runservice () {
    int pid;
    int first_serving = 1;

    /* Allocate the shared mem segment. */
    alloc_reporter(activeservice, 1);

    /* Load any allow- or deny filter files if appropriate. */
    if (ipf_loadfile (activeservice->allowfile, &(activeservice->allowchain),
		      &(activeservice->nallowchain)))
	error ("Error in allow file");
    if (ipf_loadfile (activeservice->denyfile, &(activeservice->denychain),
		      &(activeservice->ndenychain)))
	error ("Error in deny file");

    /* Go into the background. */
    if ( (pid = fork()) < 0 ) {
	/* Fork failed */
	error ("fork failure: %s", strerror(errno));
    } else if (pid) {
	/* Parent branch */
	msg ("detached as PID %d", pid);
	return;
    }

    /*** Child branch ***/
    
    /* We've forked for the first time */
    daemonized++;
    program_stage = stage_waiting;

    /* Promote verbosity. */
    flag_verbose = activeservice->verbosity;

    /* Provide configuration/limits feedback */
    config_msg();
    limits_msg();

    msg ("STARTING on port %d", activeservice->port);

    set_program_title ("crossroads - Service %s: listening",
		       activeservice->name);
    close (0);
    close (1);
    close (2);
    if ( (open ("/dev/null", O_RDONLY) < 0) ||
	 (open ("/dev/null", O_WRONLY) < 0) ||
	 (open ("/dev/null", O_WRONLY) < 0) )
	error ("failed to reopen stdin/out/err on /dev/null");
    if (setsid() < 0)
	error ("failed to become session leader");

    /* In 'forever' mode, we create a server-side socket and ask tcpserve()
     * to service it. tcpserve() will return to us if there are no back
     * ends to service the request. In that case, we close the listening
     * socket (so that new clients get denied), take a nap, and redo.
     * Of course we only start listening if we have back ends at all...
     */
    while (1) {
	/* We need at least one working back end to operate. */
	if (!backend_count()) {
	    msg ("taking a nap...");
	    sleep (SLEEP_TIME);
	    continue;
	}

	/* We have (at least) one back end. Let's run the service.
	 * Create the socket, bind to port. */
	if ( (listen_sock = make_socket (activeservice->port,
					 activeservice->bind)) < 0 ) {
	    if (!opt.sloppyportbind)
		error ("failed to listen to port %d: %s",
		       activeservice->port,
		       strerror(errno));
	    warning ("listening socket creation failed.. will retry");

	    /* We wait here for some time, to let the system regain
	     * equilibrium. Something's really afoot, so let's not retry
	     * right away. */
	    sleep (SLEEP_TIME);

	    continue;
	}

	msg ("server-side network socket: %d", listen_sock);

	/* Zero out the stats for first usage. */
	if (first_serving) {
	    first_serving = 0;
	    memset (servicereport, 0, sizeof(Servicereport));
	    /* Store our pid, so that 'crossroads stop' may kill us. */
	    lock_reporter();
	    servicereport->pid = getpid();
	    unlock_reporter();
	}

	/* Start serving the port! */
	tcpserve (listen_sock);

	/* tcpserve() returned -- must be because this service is out
	 * of back ends */
	if (shutdown (listen_sock, SHUT_RDWR))
	    warning ("failed to shut down server-side socket %d: %s",
		     listen_sock, strerror(errno));
	if (close (listen_sock))
	    warning ("failed to close server-side socket %d: %s",
		     listen_sock, strerror(errno));
    }
}
