/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* To see the command that runs, even in the wakeup handler, undefine: */
/* #define DEBUG */

int sysrun (char const *cmd) {
    char *expanded;
    int ret;
    
    if (!cmd || ! *cmd)
	return (0);

    expanded = str_expand_format (cmd);
    
#   ifdef DEBUG
    msg ("running command '%s'", expanded);
#   else    
    if (program_stage != stage_retrying)
	msg ("running command: '%s'", expanded);
#   endif

    /* Child signals are handled in system now. Also assume the right UID. */
    signal (SIGCHLD, SIG_DFL);
    uid_assume();
    ret = system (expanded);

    /* Get our own UID back and ignore child sigs. */
    uid_restore();
    signal (SIGCHLD, SIG_IGN);    

    if (ret > 0)
	warning ("command '%s' returned %d", expanded, ret);
    else if (ret < 0)
	warning ("failed to run command '%s': %s", expanded, strerror(errno));
    
#   ifdef DEBUG
    if (!ret)
	msg ("command '%s' returned %d", expanded, ret);
#    endif    

    free (expanded);

    return (ret);
}
