/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

void thruputlog (unsigned char const *buf, int len, CopyDirection dir) {
    static double d_start = 0;
    double d_now;
    int i;
    FILE *f;

    if (current_backend < 0 ||
	! activeservice->backend[current_backend].thruputfile ||
	! *activeservice->backend[current_backend].thruputfile)
	return;

    /* Initialize timer if necessary. Get current time. */
    if (!d_start)
	d_start = timeofday();
    d_now = timeofday();
	
    /* Get a handle on the reporting log. */
    if ( (! (f = fopen (activeservice->backend[current_backend].thruputfile,
			"a"))) &&
	 (! (f = fopen (activeservice->backend[current_backend].thruputfile,
			"w"))) ) {
	warning ("Cannot write %s: %s",
		 activeservice->backend[current_backend].thruputfile,
		 strerror(errno));
	return;
    }

#if HAVE_FLOCK == 1
    flock (fileno(f), LOCK_EX);
#elif HAVE_LOCKF == 1
    lockf (fileno(f), F_LOCK, 0);
#endif    
    
    /* Report the activity. */
    fprintf (f, "%7.7d %15f %c %8d ",
	     getpid(),
	     d_now - d_start,
	     dir == dir_client_to_server ? 'C' : 'B',
	     len);
    for (i = 0; i < 100 && i < len; i++)
	fputc (isprint (buf[i]) ? buf[i] : '.', f);
    fputc ('\n', f);
    
#if defined HAVE_FLOCK    
    flock (fileno(f), LOCK_UN);
#elif defined HAVE_LOCKF
    lockf (fileno(f), F_ULOCK, 0);
#endif

    fclose (f);
}
