/*************************************************************************
 * This file is part of Crosroads 1.79, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* For more verbosity of the wake up handler, undefine: */
/* #define DEBUG */

void wakeup_handler () {
    int sock, i;
    Backendavail av, prev;
    

    /* Set the stage and any signals that we want. */
    program_stage = stage_retrying;
    for (i = 0; relevant_sigs[i]; i++)
	signal (relevant_sigs[i], interrupt);

    /* Forever, until a signal kills us, or Sol explodes, causing
     * massive electromagnetic pulses that fry my CPU.
     */
    while (1) {
	/* Wait for the alarm to go off. */
	if (activeservice->rev_interval)
	    sleep (activeservice->rev_interval);
	else
	    sleep (activeservice->check_interval);

	/* Now do our stuff. */
	for (current_backend = 0;
	     current_backend < activeservice->nbackend;
	     current_backend++) {

	    /* If we're in the REVIVING mode and the back end isn't
	     * unavailable, then we don't need to check it. Also,
	     * back ends marked DOWN on purpose are skipped. */
	    prev = servicereport->backendstate[current_backend].actual_avail;
	    if ( (prev == st_down) ||
		 (activeservice->rev_interval && prev != st_unavailable) )
		continue;

#	    ifdef DEBUG	    
	    msg ("verifying back end %s, old state %s",
		 activeservice->backend[current_backend].name,
		 state_to_string(prev));
#	    endif

	    /* Mark state as WAKING */
	    lock_reporter();
	    servicereport->backendstate[current_backend].actual_avail =
		st_waking;
	    unlock_reporter();

	    if (! activeservice->check_cmd) {
		/* In TCP-only mode: do the connect */
		if ( (sock = backend_connect()) >= 0 ) {
		    close (sock);
		    av = st_available;
		} else
		    av = st_unavailable;
	    } else {
		/* In checkscript mode: do the checking */
		if (!sysrun (activeservice->check_cmd))
		    av = st_available;
		else
		    av = st_unavailable;
	    }

	    /* Set state accordingly */
	    if (prev != av) {
		if (av == st_available)
		    warning ("Backend %s of service %s has woken up",
			     activeservice->backend[current_backend].name,
			     activeservice->name);
		else
		    warning ("Backend %s of service %s has died",
			     activeservice->backend[current_backend].name,
			     activeservice->name);
	    }

	    /* Store the new state. If the new state is AVAILABLE
	     * then reset the error count (used by retries config value). */
	    lock_reporter();
	    servicereport->backendstate[current_backend].actual_avail = av;
	    if (av == st_available)
		servicereport->backendstate[current_backend].fail = 0;
	    unlock_reporter();
	}
    }
}
